Package: IONiseR
Title: Quality Assessment Tools for Oxford Nanopore MinION data
Version: 2.34.0
Authors@R: person("Mike", "Smith", email = "grimbough@gmail.com", role
        = c("aut", "cre"))
Description: IONiseR provides tools for the quality assessment of
    Oxford Nanopore MinION data. It extracts summary statistics
    from a set of fast5 files and can be used either before or
    after base calling.  In addition to standard summaries of the
    read-types produced, it provides a number of plots for
    visualising metrics relative to experiment run time or
    spatially over the surface of a flowcell.
License: MIT + file LICENSE
Depends: R (>= 3.4)
Imports: rhdf5, dplyr, magrittr, tidyr, ShortRead, Biostrings, ggplot2,
        methods, BiocGenerics, XVector, tibble, stats, BiocParallel,
        bit64, stringr, utils
VignetteBuilder: knitr
Suggests: BiocStyle, knitr, rmarkdown, gridExtra, testthat,
        minionSummaryData
biocViews: QualityControl, DataImport, Sequencing
NeedsCompilation: no
Author: Mike Smith [aut, cre]
Maintainer: Mike Smith <grimbough@gmail.com>
RoxygenNote: 6.0.1
Collate: 'IONiseR.R' 'classes.R' 'Methods-accessors.R'
        'Methods-subsetting.R' 'fast5Readers.R' 'fast5Status.R'
        'fast5readers_summary.R' 'fast5utilities.R' 'fastqProcessing.R'
        'plotting_kmers.R' 'plotting_layout.R'
        'plotting_summaryStats.R' 'processing_bam.R' 'readSummary.R'
        'squiggle.R'
git_url: https://git.bioconductor.org/packages/IONiseR
git_branch: RELEASE_3_22
git_last_commit: 25723cd
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 04:38:51 UTC; biocbuild
