# Generated by roxygen2: do not edit by hand

export(channelActivityPlot)
export(channelHeatmap)
export(fast5toFastq)
export(layoutPlot)
export(plotActiveChannels)
export(plotBaseProductionRate)
export(plotCurrentByTime)
export(plotEventRate)
export(plotKmerFrequencyCorrelation)
export(plotReadAccumulation)
export(plotReadCategoryCounts)
export(plotReadCategoryQuals)
export(plotReadTypeProduction)
export(readFast5Log)
export(readFast5Summary)
exportClasses(Fast5Summary)
exportMethods("[")
exportMethods(baseCalled)
exportMethods(eventData)
exportMethods(fastq)
exportMethods(fastq2D)
exportMethods(fastqComplement)
exportMethods(fastqTemplate)
exportMethods(length)
exportMethods(readInfo)
import(bit64)
import(ggplot2)
import(rhdf5)
importClassesFrom(ShortRead,ShortReadQ)
importFrom(BiocGenerics,width)
importFrom(BiocParallel,MulticoreParam)
importFrom(BiocParallel,bpmapply)
importFrom(BiocParallel,register)
importFrom(Biostrings,BStringSet)
importFrom(Biostrings,DNAStringSet)
importFrom(Biostrings,oligonucleotideFrequency)
importFrom(Biostrings,quality)
importFrom(ShortRead,FastqQuality)
importFrom(ShortRead,alphabetScore)
importFrom(ShortRead,append)
importFrom(ShortRead,id)
importFrom(ShortRead,sread)
importFrom(ShortRead,strand)
importFrom(ShortRead,width)
importFrom(ShortRead,writeFastq)
importFrom(XVector,compact)
importFrom(dplyr,arrange)
importFrom(dplyr,as_data_frame)
importFrom(dplyr,count)
importFrom(dplyr,data_frame)
importFrom(dplyr,filter)
importFrom(dplyr,funs)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,order_by)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,summarise_each)
importFrom(dplyr,transmute)
importFrom(dplyr,with_order)
importFrom(magrittr,"%>%")
importFrom(methods,.hasSlot)
importFrom(methods,initialize)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,setClass)
importFrom(methods,setGeneric)
importFrom(methods,setMethod)
importFrom(stats,cor)
importFrom(stats,median)
importFrom(stringr,str_detect)
importFrom(stringr,str_length)
importFrom(stringr,str_match)
importFrom(stringr,str_replace)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,gather)
importFrom(tidyr,spread)
importFrom(utils,packageVersion)
