# IONiseR

IONiseR provides tools for the quality assessment of
    Oxford Nanopore MinION data. It extracts summary statistics
    from a set of fast5 files and can be used either before or
    after base calling.  In addition to standard summaries of the
    read-types produced, it provides a number of plots for
    visualising metrics relative to experiment run time or
    spatially over the surface of a flowcell.

## Current Status

| Travis        | BioC           | Test Coverage |
| ------------- |-------------| -----|
| [![Build Status](https://travis-ci.org/grimbough/IONiseR.svg?branch=master)](https://travis-ci.org/grimbough/IONiseR) | [![BioC Status](https://bioconductor.org/shields/build/devel/bioc/IONiseR.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/IONiseR/) | [![Codecov](http://img.shields.io/codecov/c/github/grimbough/IONiseR.svg)](https://codecov.io/gh/grimbough/IONiseR) |

## Contact

Package Homepage: http://bioconductor.org/packages/devel/bioc/html/IONiseR.html 
Bug Reports: https://support.bioconductor.org/p/new/post/?tag_val=IONiseR.

## Funding 

Funding for continued development and maintenance of this package is provided by the German Network for Bioinformatics Infrastructure

<a href="http://www.denbi.de"><img src="https://tess.elixir-europe.org/system/content_providers/images/000/000/063/original/deNBI_Logo_rgb.jpg" width="400" align="left"></a>
